#!/bin/sh
# the next line restarts using tclsh \
exec tclsh "$0" ${1+"$@"}

#
# Copyright (C) 2022 Jonathan Certes
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program. If not, see <http://www.gnu.org/licenses/>.
#

namespace eval wireguard {
  # specific to the wireguard and desktop environment configurations:
  set theInterface "wg0"
  set theUpCommand   "pkexec wg-quick up   $theInterface"
  set theDownCommand "pkexec wg-quick down $theInterface"

  # content of the GUI:
  set isActive 0
  set theLabelText  "Le VPN n'est pas activé"
  set theButtonText "Activer"
}

##
 # \brief
 #  Executes command "ip link" to know if an interface is up.
 #
 # \details
 #  Iterates on each lines of the returned text, ignores lines starting with a
 #  space and considers the interface name as the second element in a line where
 #  the separator is ":".
 #
 # \returns
 #  1 if an interface name matches the one given as a parameter, 0 otherwise
 #
 # \param theInterfaceName
 #  name of the interface to be up
 #
proc wireguard::isActive {
  theInterfaceName
} {
  set theCode [catch {
    exec ip link
  } theText]

  foreach theLine [split $theText "\n"] {
    if { [string first " " $theLine] < 1 } {
      continue
    }
    set theInterface [lindex [split $theLine ":"] 1]
    set theInterface [string trim $theInterface]
    if { [string equal $theInterface $theInterfaceName] } {
      return 1
    }
  }

  return 0
}

##
 # \brief
 #  Updates the variables in the namespace that are used as text variables in
 #  the GUI.
 #
 # \details
 #  Calls function \ref wireguard::isActive to know if the interface is up. The
 #  interface is specified by variable \ref wireguard::theInterface.
 #
 # \returns
 #  integer: 0
 #
proc wireguard::updateWindow {
} {
  set wireguard::isActive 0
  if { [wireguard::isActive $wireguard::theInterface] } {
    set wireguard::isActive 1
  }

  if { $wireguard::isActive } {
    set wireguard::theLabelText  "Le VPN est activé"
    set wireguard::theButtonText "Désactiver"
    $::theImage configure -image "VPN_ON"
  } else {
    set wireguard::theLabelText  "Le VPN n'est pas activé"
    set wireguard::theButtonText "Activer"
    $::theImage configure -image "VPN_OFF"
  }

  return 0
}

##
 # \brief
 #  Actions when the activation button is pressed.
 #
 # \details
 #  Depending on the value of boolean \ref wireguard::isActive, executes either
 #  \ref wireguard::theUpCommand or \ref wireguard::theDownCommand to activate
 #  or deactivate wireguard. In the end, calls \ref wireguard::updateWindow.
 #
 # \returns
 #  integer 1 if an error occurred, otherwise what is returned by \ref
 #  wireguard::updateWindow
 #
proc wireguard::activationButttonCommand {
} {
  if { $wireguard::isActive } {
    set theCommand $wireguard::theDownCommand
    set theErrorMessage "La désactivation du VPN a retourné une erreur."
  } else {
    set theCommand $wireguard::theUpCommand
    set theErrorMessage "L'activation du VPN a retourné une erreur."
  }

  set theCode [catch {
    exec -- {*}$theCommand >& /tmp/wireguard
  } theText]

  if { $theCode } {
    tk_messageBox                     \
      -icon    "error"                \
      -default "ok"                   \
      -message $theErrorMessage       \
      -detail  "$theCommand\n$theText"
    return 1
  }

  return [wireguard::updateWindow]
}

#===============================================================================

package require Tk

image create photo "VPN_OFF" -data {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}

image create photo "VPN_ON" -data {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}

wm title . "Mon VPN"
wm resizable . 0 0
wm protocol . WM_DELETE_WINDOW { destroy . ; exit 0 }

set theTopFrame [frame .theTopFrame]
set theImage    [label .theTopFrame.theImage -image "VPN_OFF"]
set theLabel    [label .theTopFrame.theLabel \
                    -textvariable "wireguard::theLabelText"]
pack $theTopFrame
pack $theImage $theLabel -side left

set theBottomFrame  [frame  .theBottomFrame]
set theButton       [button .theBottomFrame.theButton             \
                        -textvariable "wireguard::theButtonText"  \
                        -command      "wireguard::activationButttonCommand"]
set theQuitButton [button .theBottomFrame.theQuitButton \
                            -text    "Fermer"           \
                            -command "exit 0"]
pack $theBottomFrame
pack configure $theButton $theQuitButton -side left

wireguard::updateWindow

